import matplotlib.pylab as plt

import numpy as np


def handle_2d_plot(
    embedding,
    kind,
    color=None,
    xlabel=None,
    ylabel=None,
    title=None,
    show_operations=False,
    annot=True,
    axis_option=None,
):
    """
    Handles the logic to perform a 2d plot in matplotlib.

    **Input**

    - embedding.md: a `whatlies.Embedding` object to plot
    - kind: what kind of plot to make, can be `scatter`, `arrow` or `text`
    - color: the color to apply, only works for `scatter` and `arrow`
    - xlabel: manually override the xlabel
    - ylabel: manually override the ylabel
    - title: optional title used for the plot
    - show_operations: setting to also show the applied operations, only works for `text`
    - axis_option: a string which is passed to `matplotlib.pyplot.axis` function.
    """
    if not isinstance(embedding, list):
        # It's a single Embedding instance
        embedding = [embedding]
    vectors = np.array([e.vector for e in embedding])
    names = [e.name if show_operations else e.orig for e in embedding]
    if kind == "scatter":
        if color is None:
            color = "steelblue"
        plt.scatter(vectors[:, 0], vectors[:, 1], c=color)
    if kind == "arrow":
        plt.scatter(vectors[:, 0], vectors[:, 1], c="white", s=0.01)
        plt.quiver(
            np.zeros_like(vectors[:, 0]),
            np.zeros_like(vectors[:, 1]),
            vectors[:, 0],
            vectors[:, 1],
            color=color,
            angles="xy",
            scale_units="xy",
            scale=1,
        )
    if (kind == "text") or annot:
        for vec, name in zip(vectors, names):
            plt.text(vec[0] + 0.01, vec[1], name)

    plt.xlabel("x" if xlabel is None else xlabel)
    plt.ylabel("y" if ylabel is None else ylabel)
    if title is not None:
        plt.title(title)
    if axis_option is not None:
        plt.axis(axis_option)



from sklearn.decomposition import PCA

from ._transformer import SklearnTransformer


class Pca(SklearnTransformer):
    """
    This transformer scales all the vectors in an [EmbeddingSet][whatlies.embeddingset.EmbeddingSet]
    by means of principal component analysis. We're using the implementation found in
    [scikit-learn](https://scikit-learn.org/stable/modules/generated/sklearn.decomposition.PCA.html)

    Arguments:
        n_components: the number of compoments to create/add
        kwargs: keyword arguments passed to the PCA from scikit-learn

    Usage:

    ```python
    from whatlies.language import SpacyLanguage
    from whatlies.transformers import Pca

    words = ["prince", "princess", "nurse", "doctor", "banker", "man", "woman",
             "cousin", "neice", "king", "queen", "dude", "guy", "gal", "fire",
             "dog", "cat", "mouse", "red", "bluee", "green", "yellow", "water",
             "person", "family", "brother", "sister"]

    lang = SpacyLanguage("en_core_web_md")
    emb = lang[words]

    emb.transform(Pca(3)).plot_interactive_matrix(0, 1, 2)
    ```
    """

    def __init__(self, n_components, **kwargs):
        super().__init__(
            PCA, f"pca_{n_components}", n_components=n_components, **kwargs
        )
